import json
import os
import sys

secrets = json.loads(sys.argv[1])
for key, value in secrets.items():
    print(f"Setting {key} ...")
    if key == "FIREBASE_SA_CREDENTIALS":
        continue
    else:
        lines = len(value.split("\n"))
        if lines > 1:
            os.system(f"echo '{key}<<EOF' >> $GITHUB_ENV")
            os.system(f"echo '{value}' >> $GITHUB_ENV")
            os.system("echo 'EOF' >> $GITHUB_ENV")
        else:
            os.system(f"echo '{key}={value}' >> $GITHUB_ENV")



import os
import sys
from itertools import product
from pathlib import Path

import cognite.airworkflow.constants as const
from cognite.airworkflow.function_deployer import FunctionDeployer
from cognite.airworkflow.master_config import get_master_config, get_parallel_deployment, write_dependency_yamls
from cognite.airworkflow.model_asset_hierarchy import ModelAssetHierarchy
from cognite.airworkflow.util import cdf, env, functions, github, yaml


def handle_asset_hierarchy():
    function_name, project = sys.argv[3].split("@")
    function_name_path = Path(function_name)
    function_path = const.FUNCTIONS_PATH / function_name_path / const.FUNCTION_REL_CONFIG_PATH
    master_config = {
        function_path.parent.parent: yaml.load_and_validate(function_path, const.CONFIG_SCHEMA_PATH, function_name)
    }
    write_dependency_yamls(master_config)
    if len(sys.argv) >= 2:
        airInfra = sys.argv[2] == "True"
    else:
        airInfra = False
    print(f"Running Model Asset Hierarchy create/update for project {project}")
    ModelAssetHierarchy(master_config, project, airInfra).create_or_update()


def handle_function_deployments():
    github_event_name = env.get_env_value("GITHUB_EVENT_NAME")
    github_ref = env.get_env_value("GITHUB_REF")
    is_delete = "IS_DELETE" in os.environ and os.environ["IS_DELETE"] == "true"
    is_pr = github_event_name == "pull_request"
    assert (
        len(sys.argv) >= 3 and sys.argv[2]
    ), "Expected a function name passed as a command line argument, but got nothing"
    function_name, project_name = sys.argv[2].split("@")
    print(f"HANDLING {github_event_name} ON {github_ref} FOR FUNCTION {function_name}")
    for deployment in get_parallel_deployment(function_name, project_name):
        FunctionDeployer(deployment, is_delete, is_pr).handle_deployment()


def handle_delete_functions():
    """
    1. Find name of all functions that should be deployed: Based on what's on master and what's in PRs
    2. List all functions currently deployed
    3. Delete the diff
    """
    _, project_to_config = get_master_config()
    for project, config in project_to_config.items():
        function_paths = [functions.get_relative_function_path(p) for p in config]
        if os.getenv("CI_PROJECT_ID") and os.getenv("GITLAB_ACCESS_TOKEN"):
            pr_refs = github.gitlab_open_mrs(str(os.environ["CI_PROJECT_ID"]), str(os.environ["GITLAB_ACCESS_TOKEN"]))
        else:
            pr_refs = github.get_open_pr_refs()
        all_functions = [functions.get_function_name(f, latest=True) for f in function_paths] + [
            functions.get_function_name(f, pr=True, ref=ref) for ref, f in list(product(pr_refs, function_paths))
        ]
        info = const.AUTHENTICATION[project]
        client = cdf.experimental_client(info.project)
        dangling = functions.list_dangling_function(client, all_functions, name_prefix=env.get_repo_name_auto())
        for function in dangling:
            functions.delete_function(client, function.external_id)


def handle_check_deployment():
    function_name, project_name = sys.argv[2].split("@")
    client = cdf.experimental_client(project_name)
    model_asset = client.assets.retrieve(external_id=function_name)
    if model_asset is None:
        print(str(True))
        return str(True)

    # check if function has been deployed
    repo_name = env.get_env_value("GITHUB_REPOSITORY")
    function_external_id = str(repo_name) + "/" + str(function_name) + ":latest"

    function_asset = client.functions.retrieve(external_id=function_external_id)
    if function_asset is None:
        print(str(True))
        return str(True)

    cdf_model_version = model_asset.metadata["modelVersion"]
    configuration = get_master_config()[1][project_name]
    key1 = [i for i in configuration.keys() if i.parts[-1] == function_name][0]
    model_version = configuration[key1].model_version
    model_version = ".".join(model_version.split(".")[:-1])
    already_deployed = str(model_version != cdf_model_version)
    print(already_deployed)
    return already_deployed


if __name__ == "__main__":
    if sys.argv[1] == "model":
        handle_asset_hierarchy()
    elif sys.argv[1] == "function":
        handle_function_deployments()
    elif sys.argv[1] == "delete":
        handle_delete_functions()
    elif sys.argv[1] == "check_deployment":
        handle_check_deployment()



import os
import shutil
import time
from pathlib import Path
from tempfile import TemporaryDirectory

from cognite.client import CogniteClient
from cognite.experimental import CogniteClient as ExperimentalClient

from cognite.airworkflow.constants import AUTHENTICATION, COGNITE_CLIENT_NAME

os.environ["COGNITE_MAX_RETRIES"] = "5"
os.environ["COGNITE_TIMEOUT"] = "120"


def data_set_id(client: CogniteClient, name: str) -> int:
    return client.data_sets.retrieve(external_id=name).id


def zip_and_upload(client: CogniteClient, code_path: Path, file_name: str, data_set_name: str) -> int:
    print(f"Uploading code from {code_path} ... ", flush=True)
    with TemporaryDirectory() as tmpdir:
        zip_path = Path(tmpdir) / "function"
        shutil.make_archive(str(zip_path), "zip", str(code_path))
        id = data_set_id(client, data_set_name)
        file = client.files.upload(
            f"{zip_path}.zip",
            name=f"{file_name}.zip",
            external_id=f"{file_name}.zip",
            overwrite=True,
            data_set_id=id,
        )
        counter = 0
        while client.files.retrieve(file.id) is None:
            time.sleep(0.5)
            counter += 1
            if counter > 5:
                break
        print(f"Upload complete. \n{file}", flush=True)
        return file.id


def does_function_exist(client: ExperimentalClient, function_name: str) -> bool:
    return bool(client.functions.retrieve(external_id=function_name))


def await_function_deployment(client: ExperimentalClient, function_name: str, max_wait: int) -> bool:
    t_end = time.time() + max_wait
    while time.time() < t_end:
        function = client.functions.retrieve(external_id=function_name)
        if function is not None:
            print(f"Function status {function.status} - Function id {function.id}")
            if function.status == "Ready":
                return True
            if function.status == "Failed":
                return False
        else:
            print(f"API returns None for {function_name}")
        time.sleep(5.0)

    return False


def experimental_client(project: str) -> ExperimentalClient:
    project_auth = AUTHENTICATION[project]
    if project_auth.oidc:
        experimental_client = ExperimentalClient(
            token_url=project_auth.token_url,
            token_client_id=project_auth.client_id,
            token_client_secret=project_auth.get_client_secret(),
            token_scopes=project_auth.scopes,
            project=project,
            base_url=project_auth.base_url,
            client_name=COGNITE_CLIENT_NAME,
        )
    else:
        experimental_client = ExperimentalClient(
            api_key=project_auth.get_client_api_key(),
            client_name=COGNITE_CLIENT_NAME,
            project=project,
            base_url=project_auth.base_url,
        )
    return experimental_client



import os


def get_env_value(name: str) -> str:
    value = os.getenv(name)
    if value is None:
        raise KeyError(f"Expected to find an environment variable named '{name}', but didn't")
    return value


def get_github_token() -> str:
    return get_env_value("GITHUB_TOKEN")


def get_repo_name_auto() -> str:
    return get_env_value("GITHUB_REPOSITORY_TOKEN")
